package Activator::Builder;
use strict;
use warnings;

=head1 NAME

Activator::Builder - output tarballs, cpan dist, rpm and deb packages for an Activator Project

=head1 TODO

implement this! use it in act-sync.pl!

=cut

1;
